//+-----------------------------------------------------------------------------+
//|                                                               Candle_MA.mq4 |
//|                                                         Gregory A. Kakhiani |
//|                                                         gkakhiani@gmail.com |
//+-----------------------------------------------------------------------------+
//Candle Moving Average                                                         |
//        .   |
//+-----------------------------------------------------------------------------+
//                    ----=   =-----                          |
//                                                                              |
//CandleType -                                             |
//---------------+--------------------------------------------------------------+
//            |                                                   | 
//---------------+--------------------------------------------------------------+
//                                    
//---------------+--------------------------------------------------------------+
//             1 | High-Open                                                    |
//             2 | High-Close                                                   |
//             3 | Open-Low                                                     |
//             4 | Close-Low                                                    |
//             5 | |Open-Close|                                                 |
//             6 | High-Low                                                     |
//             7 | Top Shade -                      |
//             8 | Open-Close -                                |
//             9 | Bottom Shade -                    |
//            10 |                                |
//            11 | Top Shade+Bottom Shade -         |
//            12 | Top Shade-Bottom Shade -          |
//---------------+--------------------------------------------------------------+
//                                                           |
//---------------+--------------------------------------------------------------+
//            13 | 100*(High-Open)/(High-Low)                                   |
//            14 | 100*(High-Close)/(High-Low)                                  |
//            15 | 100*(Open-Low)/(High-Low)                                    |
//            16 | 100*(Close-Low)/(High-Low)                                   |
//            17 | 100*(Open-Close)/(High-Low)                                  |
//            18 | 100*(|Open-Close|)/(High-Low)                                |
//            19 | 100*(Top Shade)/(High-Low)                                   |
//            20 | 100*(Bottom Shade)/(High-Low)                                |
//            21 | 100*(Top Shade+Bottom Shade)/(High-Low)                      |
//            22 | 100*(Top Shade-Bottom Shade)/(High-Low)                      |
//---------------+--------------------------------------------------------------+
//                                                                              |
//MaPeriod -                                                   |
//                                                                              |
//  MaType -   -                |
//                                                                              |
// VolMult -                                  |
//       0 -                                                  |
//       1 -                                                     |
//                                                                              |
//   scale -                      |
//avgscale -                    |
//                                                                              |
// k1...k6 -                        |
//+-----------------------------------------------------------------------------+


#property copyright "Gregory A. Kakhiani"
#property link      "gkakhiani@gmail.com"

#property indicator_separate_window
#property indicator_buffers 7
#property indicator_color1 Blue
#property indicator_color2 Red

#property indicator_color3 Silver
#property indicator_color4 Silver
#property indicator_color5 Silver
#property indicator_color6 Silver
#property indicator_color7 Silver

//---- input parameters
extern int       CandleType=6;
extern int       MaPeriod=360;
extern int       MaType=0;
extern int       VolMult=0;
extern double    scale=1.0;
extern double    avgscale=1.0;

//     
extern double k1=0.3333333333;
extern double k2=0.6666666666;
extern double k3=1.3333333333;
extern double k4=1.6666666666;
extern double k5=2.0000000000;
extern double k6=2.3333333333;

//---- buffers
double CandleBuffer[];
double MABuffer[];

//
double MABuffer1[];
double MABuffer2[];
double MABuffer3[];
double MABuffer4[];
double MABuffer5[];
double MABuffer6[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   IndicatorDigits(0); //

   //  
   SetIndexStyle(0,DRAW_HISTOGRAM,0,7);
   SetIndexBuffer(0,CandleBuffer);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,MABuffer);

   SetIndexStyle(2,DRAW_LINE,STYLE_DASHDOTDOT);
   SetIndexBuffer(2,MABuffer1);
   SetIndexStyle(3,DRAW_LINE,STYLE_DASHDOTDOT);
   SetIndexBuffer(3,MABuffer2);
   SetIndexStyle(4,DRAW_LINE,STYLE_DASHDOTDOT);
   SetIndexBuffer(4,MABuffer3);
   SetIndexStyle(5,DRAW_LINE,STYLE_DASHDOTDOT);
   SetIndexBuffer(5,MABuffer4);
   SetIndexStyle(6,DRAW_LINE,STYLE_DASHDOTDOT);
   SetIndexBuffer(6,MABuffer5);
   SetIndexStyle(7,DRAW_LINE,STYLE_DASHDOTDOT);
   SetIndexBuffer(7,MABuffer6);

   // 
   SetIndexLabel(0,"Candle:") ;
   SetIndexLabel(1,"Candle Average:") ;
   SetIndexLabel(2,"Level K1:") ;
   SetIndexLabel(3,"Level K2:") ;
   SetIndexLabel(4,"Level K3:") ;
   SetIndexLabel(5,"Level K4:") ;
   SetIndexLabel(6,"Level K5:") ;
   SetIndexLabel(7,"Level K6:") ;
   
   //
   if (CandleType>22)
    CandleType=1;
   
   //  
   string Candletext="";   
   switch(CandleType)
            {
             case 1: {//High-Open
                      Candletext="High-Open";
                     }break;
             case 2: {//High-Close
                      Candletext="High-Close";
                     }break;             
             case 3: {//Open-Low
                      Candletext="Open-Low";
                     }break;              
             case 4: {//Close-Low
                      Candletext="Close-Low";
                     }break;              
             case 5: {// |Open-Close|
                      Candletext="|Open-Close|";
                     }break;              
             case 6: {//High-Low
                      Candletext="High-Low";
                     }break; 
             case 7: {//     
                      Candletext="Top Shade";
                     }break; 
             case 8: {//Open-Close -    
                      Candletext="Open-Close";
                     }break; 
             case 9: {//     
                      Candletext="Bottom Shade";
                     }break; 
            case 10: {//      
                      Candletext="Candle/Candle Body";
                     }break; 
            case 11: {//Top Shade+Bottom Shade -      
                      Candletext="Top Shade+Bottom Shade";
                     }break; 
            case 12: {//Top Shade-Bottom Shade -      
                      Candletext="Top Shade-Bottom Shade";
                     }break; 
            case 13: {//100*(High-Open)/(High-Low)
                      Candletext="%(High-Open)";
                     }break; 
            case 14: {//100*(High-Close)/(High-Low)
                      Candletext="%(High-Close)";
                     }break; 
            case 15: {//100*(Open-Low)/(High-Low)
                      Candletext="%(Open-Low)";
                     }break; 
            case 16: {//100*(Close-Low)/(High-Low)
                      Candletext="%(Close-Low)";
                     }break; 
            case 17: {//100*(Open-Close)/(High-Low)
                      Candletext="%(Open-Close)";
                     }break; 
            case 18: {//100*(|Open-Close|)/(High-Low)
                      Candletext="%(|Open-Close|)";
                     }break; 
            case 19: {//100*(Top Shade)/(High-Low)
                      Candletext="%(Top Shade)";
                     }break; 
            case 20: {//100*(Bottom Shade)/(High-Low)
                      Candletext="%(Bottom Shade)";
                     }break; 
            case 21: {//100*(Top Shade+Bottom Shade)/(High-Low)
                      Candletext="%(Top Shade+Bottom Shade)";
                     }break; 
            case 22: {//100*(Top Shade-Bottom Shade)
                      Candletext="%(Top Shade-Bottom Shade)";
                     }break; 
            }
    
    string MaTypeText="";
    switch(MaType)
    {
     case 0:{
             MaTypeText="MODE_SMA"; 
            }break;
     case 1:{
             MaTypeText="MODE_EMA"; 
            }break;
     case 2:{
             MaTypeText="MODE_SMMA"; 
            }break;
     case 3:{
             MaTypeText="MODE_LWMA"; 
            }break;
    
    }
    
    string VolMultText="";    
    if(VolMult!=0)
     VolMultText="*Value";
     else VolMultText="NONE";

   IndicatorShortName("Candle MA("+Candletext+","+MaPeriod+","+MaTypeText+","+VolMultText+")");

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int    counted_bars=IndicatorCounted();
   int i;
   int j;
   int MaxBars=100000;
   double K=1/Point;
   double Tmp=0.0;
   double Tmp1=0.0;
   double Tmp2=0.0;   
   
//----

   for(i=0;i<=MaxBars;i++)
   {
    switch(CandleType)
            {
             case 1: {//High-Open
                      Tmp=K*(High[i]-Open[i]);
                     }break;
             case 2: {//High-Close
                      Tmp=K*(High[i]-Close[i]);
                     }break;             
             case 3: {//Open-Low
                      Tmp=K*(Open[i]-Low[i]);
                     }break;              
             case 4: {//Close-Low
                      Tmp=K*(Close[i]-Low[i]);
                     }break;              
             case 5: {// |Open-Close|
                      Tmp=K*MathAbs(Open[i]-Close[i]);
                     }break;              
             case 6: {//High-Low
                      Tmp=K*(High[i]-Low[i]);
                     }break; 
             case 7: {//     
                      if(Open[i]>Close[i])
                       Tmp=K*(High[i]-Open[i]);
                       else Tmp=K*(High[i]-Close[i]);
                     }break; 
             case 8: {//Open-Close -    
                      Tmp=K*MathAbs(Open[i]-Close[i]);
                      if((Open[i]>Close[i]))
                        Tmp=(-1)*Tmp;                        
                     }break; 
             case 9: {//     
                      if(Open[i]<Close[i])
                       Tmp=(-1)*K*(Open[i]-Low[i]);
                       else Tmp=(-1)*K*(Close[i]-Low[i]);
                     }break; 
            case 10: {//                  
                      Tmp1=MathAbs(Open[i]-Close[i]); 
                      if(Tmp1!=0)
                        Tmp=(High[i]-Low[i])/Tmp1;
                        else Tmp=0; 
                     }break; 
            case 11: {//|Top Shade+Bottom Shade| -      
                      if(Open[i]>Close[i])
                       Tmp=K*(High[i]-Open[i]);
                       else Tmp=K*(High[i]-Close[i]);
                     
                     if(Open[i]<Close[i])
                       Tmp1=K*(Open[i]-Low[i]);
                       else Tmp1=K*(Close[i]-Low[i]);

                      Tmp=Tmp+Tmp1;
                     }break;                      
            case 12: {//Top Shade-Bottom Shade -      
                      if(Open[i]>Close[i])
                       Tmp=K*(High[i]-Open[i]);
                       else Tmp=K*(High[i]-Close[i]);
                     
                     if(Open[i]<Close[i])
                       Tmp1=(-1)*K*(Open[i]-Low[i]);
                       else Tmp1=(-1)*K*(Close[i]-Low[i]);

                      Tmp=Tmp+Tmp1;
                     }break;                      
            case 13: {//100*(High-Open)/(High-Low)
                     Tmp1=High[i]-Low[i];
                     if(Tmp1==0) 
                      Tmp=100;
                      else Tmp=100*(High[i]-Open[i])/Tmp1;                     
                     }break;                      
            case 14: {//100*(High-Close)/(High-Low)
                     Tmp1=High[i]-Low[i];
                     if(Tmp1==0) 
                      Tmp=100;
                      else Tmp=100*(High[i]-Close[i])/Tmp1;                     
                     }break; 
            case 15: {//100*(Open-Low)/(High-Low)
                     Tmp1=High[i]-Low[i];
                     if(Tmp1==0) 
                      Tmp=100;
                      else Tmp=100*(Open[i]-Low[i])/Tmp1;                     
                     }break; 
            case 16: {//100*(Close-Low)/(High-Low)
                     Tmp1=High[i]-Low[i];
                     if(Tmp1==0) 
                      Tmp=100;
                      else Tmp=100*(Close[i]-Low[i])/Tmp1;                     
                     }break; 
            case 17: {//100*(Open-Close)/(High-Low)
                     Tmp1=High[i]-Low[i];
                     if(Tmp1==0) 
                      Tmp=100;
                      else {                                                 
                            Tmp=100*MathAbs(Open[i]-Close[i])/Tmp1;
                            if((Open[i]>Close[i]))
                            Tmp=(-1)*Tmp;         
                           }               
                     }break;                                                  
            case 18: {//100*(|Open-Close|)/(High-Low)
                     Tmp1=High[i]-Low[i];
                     if(Tmp1==0) 
                      Tmp=100;
                      else Tmp=100*MathAbs(Open[i]-Close[i])/Tmp1;                     
                     }break; 
            case 19: {//100*(Top Shade)/(High-Low)
                      Tmp1=High[i]-Low[i];
                      if(Open[i]>Close[i])
                       Tmp=100*(High[i]-Open[i]);
                       else Tmp=100*(High[i]-Close[i]);
                     
                     if(Tmp1==0) 
                      Tmp=100;
                      else Tmp=Tmp/Tmp1;                     
                     }break; 
            case 20: {//100*(Bottom Shade)/(High-Low)
                      Tmp1=High[i]-Low[i];
                      if(Open[i]<Close[i])
                       Tmp=(-1)*100*(Open[i]-Low[i]);
                       else Tmp=(-1)*100*(Close[i]-Low[i]);
                     
                     if(Tmp1==0) 
                      Tmp=100;
                      else Tmp=Tmp/Tmp1;                     
                     }break; 
           case 21: {//100*(Top Shade+Bottom Shade)/(High-Low)
                     Tmp2=High[i]-Low[i];
                     if(Open[i]>Close[i])
                       Tmp=100*(High[i]-Open[i]);
                       else Tmp=100*(High[i]-Close[i]);
                     
                     if(Open[i]<Close[i])
                       Tmp1=100*(Open[i]-Low[i]);
                       else Tmp1=100*(Close[i]-Low[i]);

                      Tmp=Tmp+Tmp1;
                      if(Tmp2==0) 
                      Tmp=100;
                      else Tmp=Tmp/Tmp2;     
                     }break; 
           case 22: {//100*(Top Shade-Bottom Shade)/(High-Low)
                     Tmp2=High[i]-Low[i];
                     if(Open[i]>Close[i])
                       Tmp=100*(High[i]-Open[i]);
                       else Tmp=100*(High[i]-Close[i]);
                     
                     if(Open[i]<Close[i])
                       Tmp1=100*(Open[i]-Low[i]);
                       else Tmp1=100*(Close[i]-Low[i]);

                      Tmp=Tmp-Tmp1;
                      if(Tmp2==0) 
                      Tmp=100;
                      else Tmp=Tmp/Tmp2;     
                     }break;                     
            }

    //  
    if(VolMult!=0)
     Tmp=Tmp*Volume[i];
    CandleBuffer[i]=Tmp*scale; //        
   }
   
   for(i=0;i<=MaxBars;i++)
    {
      //  
      MABuffer[i]=iMAOnArray(CandleBuffer,0,MaPeriod,0,MaType,i)*avgscale;
     
     //   
     MABuffer1[i]=k1*MABuffer[i];
     MABuffer2[i]=k2*MABuffer[i];
     MABuffer3[i]=k3*MABuffer[i];
     MABuffer4[i]=k4*MABuffer[i];
     MABuffer5[i]=k5*MABuffer[i];
     MABuffer6[i]=k6*MABuffer[i];     
    }
//----
   return(0);
  }
//+------------------------------------------------------------------+